Imports System.Text
Imports System.IO.Ports
Imports System.Threading
Imports CrustCrawler.Interface.Robotis

Public Class SmartArm
    Implements IDisposable

    ' Keep track of when the object is disposed.
    Protected m_Disposed As Boolean = False

    Protected WithEvents m_U2D As Dynamixel

#Region " Constructor/Destructor "
    ''' <summary>
    ''' Constructs the SmartArm object.
    ''' </summary>
    Public Sub New(ByVal ComPort As String)

        m_U2D = New Dynamixel(ComPort, 1000000)
        m_U2D.OpenPort()

    End Sub

    Protected Overridable Sub Dispose(ByVal disposing As Boolean)

        If Not Me.m_Disposed Then
            If disposing Then
                ' Insert code to free unmanaged resources
            End If

            ' Free shared resources
            If m_U2D.PortIsOpen Then
                m_U2D.ClosePort()
            End If
            If Not m_U2D Is Nothing Then
                m_U2D = Nothing
            End If
        End If

        Me.m_Disposed = True

    End Sub

    ' Do not change or add Overridable to these methods.
    ' Put cleanup code in Dispose(ByVal disposing As Boolean).
    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

#End Region

#Region "Smart Arm Methods"

    Public Sub SetBase(ByVal Position As Integer, ByVal Speed As Integer, Optional ByVal Delay As Integer = 0)

        m_U2D.WriteRegister(1, Dynamixel.DynamixelRegister.R32_Speed_L, SplitInteger(Speed))
        m_U2D.WriteRegister(1, Dynamixel.DynamixelRegister.R30_GoalPosition_L, SplitInteger(Position))
        DoDelay(1, Delay)

    End Sub

    Public Sub LockBase(ByVal Enable As Boolean)
        TorqueEnable(1, Enable)
    End Sub

    Public Function IsBaseMoving() As Boolean
        Return IsMoving(1)
    End Function

    Public Sub SetElbow(ByVal Position As Integer, ByVal Speed As Integer, Optional ByVal Delay As Integer = 0)

        Dim Params As New List(Of Byte)

        Params.Add(3)
        Params.AddRange(SplitInteger(Position))
        Params.AddRange(SplitInteger(Speed))

        Dim inversePos As Integer = (511 - Position) + 511
        Params.Add(2)
        Params.AddRange(SplitInteger(inversePos))
        Params.AddRange(SplitInteger(Speed))
        m_U2D.SyncWrite(Dynamixel.DynamixelRegister.R30_GoalPosition_L, 2, Params.ToArray)
        DoDelay(3, Delay)

    End Sub

    Public Sub LockElbow(ByVal Enable As Boolean)
        TorqueEnable(2, Enable)
        TorqueEnable(3, Enable)
    End Sub

    Public Function IsElbowMoving() As Boolean
        Return IsMoving(3)
    End Function

    Public Sub SetWrist(ByVal Position As Integer, ByVal Speed As Integer, Optional ByVal Delay As Integer = 0)

        Dim wristParams As New List(Of Byte)

        wristParams.Add(4)
        wristParams.AddRange(SplitInteger(Position))
        wristParams.AddRange(SplitInteger(Speed))

        Dim inversePos As Integer = (511 - Position) + 511
        wristParams.Add(5)
        wristParams.AddRange(SplitInteger(inversePos))
        wristParams.AddRange(SplitInteger(Speed))
        m_U2D.SyncWrite(Dynamixel.DynamixelRegister.R30_GoalPosition_L, 2, wristParams.ToArray)
        DoDelay(4, Delay)

    End Sub

    Public Sub LockWrist(ByVal Enable As Boolean)
        TorqueEnable(4, Enable)
        TorqueEnable(5, Enable)
    End Sub

    Public Function IsWristMoving() As Boolean
        Return IsMoving(4)
    End Function

    Public Sub SetWristRotate(ByVal Position As Integer, ByVal Speed As Integer, Optional ByVal Delay As Integer = 0)

        m_U2D.WriteRegister(6, Dynamixel.DynamixelRegister.R32_Speed_L, SplitInteger(Speed))
        m_U2D.WriteRegister(6, Dynamixel.DynamixelRegister.R30_GoalPosition_L, SplitInteger(Position))
        DoDelay(6, Delay)

    End Sub

    Public Sub LockWristRotate(ByVal Enable As Boolean)
        TorqueEnable(6, Enable)
    End Sub

    Public Function IsWristRotateMoving() As Boolean
        Return IsMoving(6)
    End Function

    Public Sub SetGripper(ByVal Position As Integer, ByVal Speed As Integer, Optional ByVal Delay As Integer = 0)

        m_U2D.WriteRegister(7, Dynamixel.DynamixelRegister.R32_Speed_L, SplitInteger(Speed))
        If Position < 511 Then
            m_U2D.WriteRegister(7, Dynamixel.DynamixelRegister.R30_GoalPosition_L, SplitInteger(511))
        ElseIf Position > 750 Then
            m_U2D.WriteRegister(7, Dynamixel.DynamixelRegister.R30_GoalPosition_L, SplitInteger(750))
        Else
            m_U2D.WriteRegister(7, Dynamixel.DynamixelRegister.R30_GoalPosition_L, SplitInteger(Position))
        End If
        DoDelay(7, Delay)

    End Sub

    Public Sub LockGripper(ByVal Enable As Boolean)
        TorqueEnable(7, Enable)
    End Sub

    Public Function IsGripperMoving() As Boolean
        Return IsMoving(7)
    End Function

    Public Sub LockAll(ByVal Enable As Boolean)

        LockBase(Enable)
        LockElbow(Enable)
        LockWrist(Enable)
        LockWristRotate(Enable)
        LockGripper(Enable)

    End Sub

#End Region

#Region "Properties"

    Public ReadOnly Property GetBasePosition() As Integer
        Get
            Dim value As Byte() = m_U2D.ReadRegister(1, Dynamixel.DynamixelRegister.R36_CurrentPosition_L)
            Return (value(6) * 256) + value(5)
        End Get
    End Property

    Public ReadOnly Property GetElbowPosition() As Integer
        Get
            Dim value As Byte() = m_U2D.ReadRegister(3, Dynamixel.DynamixelRegister.R36_CurrentPosition_L)
            Return (value(6) * 256) + value(5)
        End Get
    End Property

    Public ReadOnly Property GetWristPosition() As Integer
        Get
            Dim value As Byte() = m_U2D.ReadRegister(4, Dynamixel.DynamixelRegister.R36_CurrentPosition_L)
            Return (value(6) * 256) + value(5)
        End Get
    End Property

    Public ReadOnly Property GetWristRotatePosition() As Integer
        Get
            Dim value As Byte() = m_U2D.ReadRegister(6, Dynamixel.DynamixelRegister.R36_CurrentPosition_L)
            Return (value(6) * 256) + value(5)
        End Get
    End Property

    Public ReadOnly Property GetGripperPosition() As Integer
        Get
            Dim value As Byte() = m_U2D.ReadRegister(7, Dynamixel.DynamixelRegister.R36_CurrentPosition_L)
            Return (value(6) * 256) + value(5)
        End Get
    End Property

    Public ReadOnly Property U2dDynamixel() As Dynamixel
        Get
            Return m_U2D
        End Get
    End Property

#End Region

#Region "Helper Methods"

    Private Sub DoDelay(ByVal ID As Integer, ByVal Delay As Integer)

        If Delay > 0 Then
            Thread.Sleep(Delay)
        ElseIf Delay = -1 Then
            Do While IsMoving(ID)
            Loop
        End If

    End Sub

    Private Function IsMoving(ByVal ID As Integer) As Boolean

        Dim value As Byte() = m_U2D.ReadRegister(ID, Dynamixel.DynamixelRegister.R46_Moving)
        If value(5) = 1 Then
            Return True
        Else
            Return False
        End If

    End Function

    Private Sub TorqueEnable(ByVal ID As Integer, ByVal Enable As Boolean)

        Dim en() As Byte = {0}
        If Enable Then
            en(0) = 1
        End If
        m_U2D.WriteRegister(ID, Dynamixel.DynamixelRegister.R24_EnableTorque, en)

    End Sub

    Private Function SplitInteger(ByVal arg As Integer) As Byte()

        Dim LowHigh(1) As Byte
        LowHigh(0) = CByte((arg And &HFF))
        LowHigh(1) = CByte((arg And &HFF00) / &H100)
        Return LowHigh

    End Function

#End Region

    Public Overrides Function Equals(ByVal obj As Object) As Boolean
        Return MyBase.Equals(obj)
    End Function

End Class
