#Region " Copyright "
'***************************************************************************************************'
'
'   File....... DynamixelControlTable.vb
'   Purpose.... Dynamixel Control Table Object
'   Author..... CrustCrawler Inc. (Mike Gebhard)
'   Help....... http://forum.crustcrawler.com/
'   Started.... 9/30/2007
'   Updated.... 
'   Version.... 001
'
'   Copyright CrustCrawler, Inc.
'***************************************************************************************************
#End Region

#Region " Maintenance Log "
'***************************************************************************************************'
' Description:
' The Dynamixel control table class holds the values of a Dynamixel control table.  See the 
'   Dynamixel manual for details.
'
'
' Version           Programmer          Release Date            Details
'--------------------------------------------------------------------------------------------------
' 001               Mike Gebhard        10/6/2007               Original Program
'
'***************************************************************************************************
#End Region

Public Class DynamixelControlTable

    'Events
    Public Event DynamixelControlTableChange As DynamixelControlTableChangeHandler

    'Save the error response
    Private _error As Byte

    'Control table array
    Private _registerArray() As Byte

    'Dynamixel ID
    Private _id As Byte



#Region " Constructor "

    'Public Sub New()
    '    'Default constructor
    'End Sub

    Public Sub New(ByVal controlTableArray() As Byte)
        _registerArray = GetRegisterArray(controlTableArray)
        _error = controlTableArray(4)
        _id = _registerArray(3)
    End Sub

    'Get only the control table values from the response stream
    '   The first 5 bytes in the response stream are FF, FF, ID, Length, & Error
    '   The last one is the checksum
    Private Function GetRegisterArray(ByVal dcta As Byte()) As Byte()
        Dim cta(50) As Byte
        For i As Integer = 0 To 49
            cta(i) = dcta(i + 5)
        Next
        Return cta
    End Function

#End Region


#Region " Properties "
    '=========================================================================
    ' The properties are named after the Dyanmixel control table registers
    '   I made a couple of name changes but nothing substantial.
    '
    '
    '=========================================================================

    Public ReadOnly Property ModelNumber() As Integer
        Get
            Return ConvertToWord(_registerArray(1), _registerArray(0))
        End Get

    End Property


    Public ReadOnly Property FirmwareVersion() As Byte
        Get
            Return _registerArray(2)
        End Get
    End Property


    Public Property ID() As Byte
        Get
            Return _registerArray(3)
        End Get
        Set(ByVal value As Byte)
            If Not value = _registerArray(3) Then
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R03_Id, CByte(value)))
                _registerArray(3) = value
            End If
        End Set
    End Property


    Public Property BaudRate() As Dynamixel.baudrate
        Get
            Return CType([Enum].Parse(GetType(Dynamixel.baudrate), _registerArray(4)), Dynamixel.baudrate)
        End Get
        Set(ByVal value As Dynamixel.baudrate)
            If Not value = _registerArray(4) Then
                _registerArray(4) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R04_Baud, CByte(value)))
            End If
        End Set
    End Property


    Public Property ReturnDelay() As Byte
        Get
            Return _registerArray(5)
        End Get
        Set(ByVal value As Byte)
            If Not value = _registerArray(5) Then
                _registerArray(5) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R05_ReturnDelayX2uSec, CByte(value)))
            End If
        End Set
    End Property


    Public Property CW_AngleLimit() As Integer
        Get
            Return ConvertToWord(_registerArray(7), _registerArray(6))
        End Get
        Set(ByVal value As Integer)
            If Not value = ConvertToWord(_registerArray(7), _registerArray(6)) Then
                'Low byte
                _registerArray(6) = CByte((value And &HFF))
                'High byte
                _registerArray(7) = CByte((value And &HFF00) / &H100)
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R06_ClockwiseAngleLimit_L, CByte(_registerArray(6)), CByte(_registerArray(7))))
            End If
        End Set
    End Property


    Public Property CCW_AngleLimit() As Integer
        Get
            Return ConvertToWord(_registerArray(9), _registerArray(8))
        End Get
        Set(ByVal value As Integer)
            If Not value = ConvertToWord(_registerArray(7), _registerArray(6)) Then
                'Low byte
                _registerArray(8) = CByte((value And &HFF))
                'High byte
                _registerArray(9) = CByte((value And &HFF00) / &H100)
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R06_ClockwiseAngleLimit_L, CByte(_registerArray(8)), CByte(_registerArray(9))))
            End If

        End Set
    End Property


    Public ReadOnly Property R10_Reserved() As Byte
        Get
            Return _registerArray(10)
        End Get
    End Property


    Public Property MaxTemperature() As Byte
        Get
            Return _registerArray(11)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(11) Then
                _registerArray(11) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R11_TemperatureLimitC, CByte(value)))
            End If
        End Set
    End Property


    Public Property MinVoltage() As Byte
        Get
            Return _registerArray(12)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(12) Then
                _registerArray(12) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R12_LowVoltageLimit, CByte(value)))
            End If
        End Set
    End Property



    Public Property MaxVoltage() As Byte
        Get
            Return _registerArray(13)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(13) Then
                _registerArray(13) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R13_HighVoltageLimit, CByte(value)))
            End If
        End Set
    End Property


    Public Property MaxTorque() As Integer
        Get
            Return ConvertToWord(_registerArray(15), _registerArray(14))
        End Get
        Set(ByVal value As Integer)
            If Not value = ConvertToWord(_registerArray(15), _registerArray(14)) Then
                'Low byte
                _registerArray(14) = CByte((value And &HFF))
                'High byte
                _registerArray(15) = CByte((value And &HFF00) / &H100)
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R14_MaxTorque_L, CByte(_registerArray(14)), CByte(_registerArray(15))))
            End If
        End Set
    End Property


    Public Property StatusReturnLevel() As Byte
        Get
            Return _registerArray(16)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(16) Then
                _registerArray(16) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R16_StatusReturnLevel, CByte(value)))
            End If
        End Set
    End Property

    Public Property LEDAlarm() As Byte
        Get
            Return _registerArray(17)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(17) Then
                _registerArray(17) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R17_LedAlarm, CByte(value)))
            End If
        End Set
    End Property


    Public Property AlarmShutdown() As Byte
        Get
            Return _registerArray(18)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(18) Then
                _registerArray(18) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R18_ShutdownAlarm, CByte(value)))
            End If
        End Set
    End Property


    Public ReadOnly Property R19_Reserved() As Byte
        Get
            Return _registerArray(19)
        End Get
    End Property


    Public ReadOnly Property DownCalibration() As Integer
        Get
            Return ConvertToWord(_registerArray(21), _registerArray(20))
        End Get
    End Property


    Public ReadOnly Property UpCalibration() As Integer
        Get
            Return ConvertToWord(_registerArray(23), _registerArray(22))
        End Get
    End Property


    Public Property EnableTorque() As Byte
        Get
            Return _registerArray(24)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(24) Then
                _registerArray(24) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R24_EnableTorque, CByte(value)))
            End If
        End Set
    End Property


    Public Property LED() As Byte
        Get
            Return _registerArray(25)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(25) Then
                _registerArray(25) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R25_LED, CByte(value)))
            End If
        End Set
    End Property


    Public Property CW_ComplianceMargin() As Byte
        Get
            Return _registerArray(26)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(26) Then
                _registerArray(26) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R26_ClockwiseComplienceMargin, CByte(value)))
            End If
        End Set
    End Property


    Public Property CCW_ComplianceMargin() As Byte
        Get
            Return _registerArray(27)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(27) Then
                _registerArray(27) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R27_CounterClockwiseComplienceMargin, CByte(value)))
            End If
        End Set
    End Property


    Public Property CW_ComplianceSlope() As Byte
        Get
            Return _registerArray(28)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(28) Then
                _registerArray(28) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R28_ClockwiseComplienceSlope, CByte(value)))
            End If
        End Set
    End Property


    Public Property CCW_ComplianceSlope() As Byte
        Get
            Return _registerArray(29)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(29) Then
                _registerArray(29) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R29_CounterClockwiseComplienceSlope, CByte(value)))
            End If
        End Set
    End Property


    Public Property GoalPosition() As Integer
        Get
            Return ConvertToWord(_registerArray(31), _registerArray(30))
        End Get
        Set(ByVal value As Integer)
            If Not value = ConvertToWord(_registerArray(31), _registerArray(30)) Then
                'Low byte
                _registerArray(30) = CByte((value And &HFF))
                'High byte
                _registerArray(31) = CByte((value And &HFF00) / &H100)
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R30_GoalPosition_L, _registerArray(30), _registerArray(31)))
            End If
        End Set
    End Property


    Public Property Speed() As Integer
        Get
            Return ConvertToWord(_registerArray(33), _registerArray(32))
        End Get
        Set(ByVal value As Integer)
            If Not value = ConvertToWord(_registerArray(33), _registerArray(32)) Then
                'Low byte
                _registerArray(32) = CByte((value And &HFF))
                'High byte
                _registerArray(33) = CByte((value And &HFF00) / &H100)
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R32_Speed_L, _registerArray(32), _registerArray(33)))
            End If
        End Set
    End Property



    Public Property TorqueLimit() As Integer
        Get
            Return ConvertToWord(_registerArray(35), _registerArray(34))
        End Get
        Set(ByVal value As Integer)
            If Not value = ConvertToWord(_registerArray(35), _registerArray(34)) Then
                'Low byte
                _registerArray(34) = CByte((value And &HFF))
                'High byte
                _registerArray(35) = CByte((value And &HFF00) / &H100)
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R34_TourqueLimit_L, _registerArray(34), _registerArray(35)))
            End If
        End Set
    End Property



    Public ReadOnly Property CurrentPosition() As Integer
        Get
            Return ConvertToWord(_registerArray(37), _registerArray(36))
        End Get
    End Property


    Public ReadOnly Property CurrentSpeed() As Integer
        Get
            Return ConvertToWord(_registerArray(39), _registerArray(38))
        End Get
    End Property


    Public ReadOnly Property CurrentLoad() As Integer
        Get
            Return ConvertToWord(_registerArray(41), _registerArray(40))
        End Get
    End Property


    Public ReadOnly Property CurrentVoltage() As Byte
        Get
            Return _registerArray(42)
        End Get
    End Property


    Public ReadOnly Property CurrentTemperature() As Byte
        Get
            Return _registerArray(43)
        End Get
    End Property


    Public Property RegisterInstruction() As Byte
        Get
            Return _registerArray(44)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(44) Then
                _registerArray(44) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R44_RegisteredInstruction, CByte(value)))
            End If
        End Set
    End Property


    Public ReadOnly Property R45_Reserved() As Byte
        Get
            Return _registerArray(45)
        End Get
    End Property


    Public ReadOnly Property Moving() As Byte
        Get
            Return _registerArray(46)
        End Get
    End Property


    Public Property Lock() As Byte
        Get
            Return _registerArray(47)
        End Get
        Set(ByVal value As Byte)
            If value <> _registerArray(47) Then
                _registerArray(47) = value
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R47_Lock, CByte(value)))
            End If
        End Set
    End Property


    Public Property Punch() As Integer
        Get
            Return ConvertToWord(_registerArray(49), _registerArray(48))
        End Get
        Set(ByVal value As Integer)
            If Not value = ConvertToWord(_registerArray(49), _registerArray(48)) Then
                'Low byte
                _registerArray(48) = CByte((value And &HFF))
                'High byte
                _registerArray(49) = CByte((value And &HFF00) / &H100)
                OnDynamixelControlTableChange(New DynamixelControlTableArgs(_id, Dynamixel.DynamixelRegister.R48_Punch_L, _registerArray(48), _registerArray(49)))
            End If

        End Set
    End Property


    Public ReadOnly Property [Error]() As Byte
        Get
            Return _error
        End Get
    End Property


    Public ReadOnly Property RegisterArray() As Byte()
        Get
            Return _registerArray
        End Get
    End Property


#End Region


#Region " Helper Methods "

    'Takes 2 bytes high and low and converts it to an integer value
    Private Function ConvertToWord(ByVal HighByte As Byte, ByVal LowBye As Byte) As Integer
        Return CInt(String.Format("&H{0}{1}", Conversion.Hex(HighByte), PadLowByte(Conversion.Hex(LowBye))))
    End Function

    'Pad the low byte with a zero ir it is one character long.
    Private Function PadLowByte(ByVal lowByte As String) As String
        If lowByte.Length = 1 Then
            Return 0 & lowByte
        End If
        Return lowByte
    End Function

#End Region


#Region " Events "

    'If any property is updated then this event is raised.  DynamixelControlTableArgs
    '   contains the register address (enum) to update.
    Private Sub OnDynamixelControlTableChange(ByVal e As DynamixelControlTableArgs)
        RaiseEvent DynamixelControlTableChange(Me, e)
    End Sub 'OnDynamixelControlTableChange

#End Region


#Region " Print "
    'This method displays a Dynamixel control table
    Public Function PrintRegister() As System.Text.StringBuilder
        Dim strBuilder As New System.Text.StringBuilder()
        Try
            strBuilder.AppendLine(String.Format("Model Number: {0}{0}{1}", vbTab, CInt("&H" & Conversion.Hex(_registerArray(1)) & Conversion.Hex(_registerArray(0)))))
            strBuilder.AppendLine(String.Format("Firmware Version: {0}{0}{1}", vbTab, _registerArray(2)))
            strBuilder.AppendLine(String.Format("Dynamixel ID: {0}{0}{1}", vbTab, ID))
            strBuilder.AppendLine(String.Format("Baud Rate: {0}{0}{1}", vbTab, BaudRate))
            strBuilder.AppendLine(String.Format("Return Delay: {0}{0}{1}", vbTab, ReturnDelay))
            strBuilder.AppendLine(String.Format("CW AngleLimit: {0}{0}{1}", vbTab, CW_AngleLimit))
            strBuilder.AppendLine(String.Format("CCW AngleLimit: {0}{0}{1}", vbTab, CCW_AngleLimit))
            strBuilder.AppendLine(String.Format("Max Temperature: {0}{0}{1}", vbTab, MaxTemperature))
            strBuilder.AppendLine(String.Format("Min Voltage: {0}{0}{1}", vbTab, MinVoltage))
            strBuilder.AppendLine(String.Format("Max Voltage: {0}{0}{1}", vbTab, MaxVoltage))
            strBuilder.AppendLine(String.Format("Max Torque: {0}{0}{1}", vbTab, MaxTorque))
            strBuilder.AppendLine(String.Format("Status Return Level: {0}{1}", vbTab, StatusReturnLevel))
            strBuilder.AppendLine(String.Format("Alarm LED: {0}{0}{1}", vbTab, LEDAlarm))
            strBuilder.AppendLine(String.Format("Alarm Shutdown: {0}{0}{1}", vbTab, AlarmShutdown))
            strBuilder.AppendLine(String.Format("Down Calibration: {0}{0}{1}", vbTab, DownCalibration))
            strBuilder.AppendLine(String.Format("Up Calibration: {0}{0}{1}", vbTab, UpCalibration))
            strBuilder.AppendLine(String.Format("Torque Enabled: {0}{0}{1}", vbTab, EnableTorque))
            strBuilder.AppendLine(String.Format("LED: {0}{0}{0}{1}", vbTab, LED))
            strBuilder.AppendLine(String.Format("CW ComplianceMargin: {0}{1}", vbTab, CW_ComplianceMargin))
            strBuilder.AppendLine(String.Format("CCW ComplianceMargin: {0}{1}", vbTab, CCW_ComplianceMargin))
            strBuilder.AppendLine(String.Format("CW_ComplianceSlope: {0}{1}", vbTab, CW_ComplianceSlope))
            strBuilder.AppendLine(String.Format("CCW_ComplianceSlope: {0}{1}", vbTab, CCW_ComplianceSlope))
            strBuilder.AppendLine(String.Format("Goal Position: {0}{0}{1}", vbTab, GoalPosition))
            strBuilder.AppendLine(String.Format("Speed: {0}{0}{0}{1}", vbTab, Speed))
            strBuilder.AppendLine(String.Format("Torque Limit: {0}{0}{1}", vbTab, TorqueLimit))
            strBuilder.AppendLine(String.Format("Current Position: {0}{0}{1}", vbTab, CurrentPosition))
            strBuilder.AppendLine(String.Format("Current Speed: {0}{0}{1}", vbTab, CurrentSpeed))
            strBuilder.AppendLine(String.Format("Current Load: {0}{0}{1}", vbTab, CurrentLoad))
            strBuilder.AppendLine(String.Format("Current Voltage: {0}{0}{1}", vbTab, CurrentVoltage))
            strBuilder.AppendLine(String.Format("Current Temperature: {0}{1}", vbTab, CurrentTemperature))
            strBuilder.AppendLine(String.Format("Register Instruction: {0}{1}", vbTab, RegisterInstruction))
            strBuilder.AppendLine(String.Format("Moving: {0}{0}{0}{1}", vbTab, Moving))
            strBuilder.AppendLine(String.Format("Lock: {0}{0}{0}{1}", vbTab, Lock))
            strBuilder.AppendLine(String.Format("Punch: {0}{0}{0}{1}", vbTab, Punch))
            Return strBuilder
        Catch ex As Exception
            strBuilder.AppendLine(String.Format("Error reading control table{0}{1}", vbCrLf, ex.ToString))
        End Try
        Return strBuilder.AppendLine(String.Format("Noting to print!{} Check DynamixelControlTable.PrintRegister for errors.", vbCrLf))
    End Function

#End Region

End Class
