#Region " Copyright "
'***************************************************************************************************'
'
'   File....... Usb2DynamixelEventArgs.vb
'   Purpose.... Interface with the Robtis Usb2Dynamixel
'   Author..... CrustCrawler Inc. (Mike Gebhard)
'   Help....... http://forum.crustcrawler.com/
'   Started.... 7/13/2007
'   Updated.... 
'   Version.... 1.0.0.0
'
'   Copyright CrustCrawler, Inc.
'
'
'
'
'***************************************************************************************************
#End Region

#Region " Maintenance Log "
'***************************************************************************************************'
' Description:
' Classes and delegates to handle Dynamixel communications.
'
'
' Version           Programmer          Release Date            Details
'--------------------------------------------------------------------------------------------------
' 001               Mike Gebhard        7/13/2007               Original Program
'
'***************************************************************************************************
#End Region

Imports System.Collections.Generic

'Delegates
Public Delegate Sub DynamixelErrorHandler(ByVal sender As Object, ByVal e As DynamixelErrorArgs)
Public Delegate Sub DynamixelWriteHandler(ByVal sender As Object, ByVal e As DynamixelWriteArgs)
Public Delegate Sub DynamixelReadHandler(ByVal sender As Object, ByVal e As DynamixelReadArgs)
Public Delegate Sub DynamixelControlTableChangeHandler(ByVal sender As Object, ByVal e As DynamixelControlTableArgs)


#Region " DynamixelErrorArgs "

Public Class DynamixelErrorArgs : Inherits EventArgs

    Private Enum ErrorType As Byte
        Network = &H80
        Instruction = &H40
        Overload = &H20
        Checksum = &H10
        Range = &H8
        Overheating = &H4
        AngleLimit = &H2
        InputVoltage = &H1
    End Enum


    Private _errorDescription As New List(Of String)


    Public ReadOnly Property ErrorDescription() As List(Of String)
        Get
            Return _errorDescription
        End Get
    End Property

    Public ReadOnly Property ErrorCount() As Integer
        Get
            Return _errorDescription.Count
        End Get
    End Property


    Public Sub New(ByVal errorCode As Integer)
        Try
            If (errorCode And ErrorType.Network) > 0 Then
                _errorDescription.Add("Warning: No return packet detected." & vbCrLf)
            End If
            If (errorCode And ErrorType.Instruction) > 0 Then
                _errorDescription.Add("Unidentified instruction or an Action instruction was sent without first issuing a Reg_Write instruction." & vbCrLf)
            End If
            If (errorCode And ErrorType.Overload) > 0 Then
                _errorDescription.Add("Maximum torque reached.  Dynamixel cannot handle the applied load. (0x0E 0x0F)" & vbCrLf)
            End If
            If (errorCode And ErrorType.Checksum) > 0 Then
                _errorDescription.Add("Incorrect checksum." & vbCrLf)
            End If
            If (errorCode And ErrorType.Range) > 0 Then
                _errorDescription.Add("The instruction issued is out of the definded range.  Value instructions are 0x01 to 0x06 and 0x83" & vbCrLf)
            End If
            If (errorCode And ErrorType.Overheating) > 0 Then
                _errorDescription.Add("Internal dynamixel temperature is above the operating temperature defined in the control table. (0x0B)" & vbCrLf)
            End If
            If (errorCode And ErrorType.AngleLimit) > 0 Then
                _errorDescription.Add("The goal position is set outside the clockwise angle limit and counterclockwise angle limit. (0x06 0x07, 0x08 0x09)" & vbCrLf)
            End If
            If (errorCode And ErrorType.InputVoltage) > 0 Then
                _errorDescription.Add("Internal dynamixel voltage is above the operating voltage defined in the control table. (0x0C)" & vbCrLf)
            End If
        Catch ex As Exception
            Throw New ApplicationException("Error in Usb2DynamixelErrorArgs - Oops, sorry about that :)~", ex)
        End Try
    End Sub

End Class 'Usb2DynamixelEventArgs

#End Region

#Region " DynamixelWriteArgs "

Public Class DynamixelWriteArgs : Inherits EventArgs

    Private _outputBuffer As String = Nothing


    Public Sub New(ByVal buff As Byte())
        If Not buff Is Nothing AndAlso buff.Length > 0 Then
            For Each b As Byte In buff
                _outputBuffer += IIf(Hex(b).Length = 1, "0" & Hex(b), Hex(b)) & " "
            Next
        End If
    End Sub


    Public Property OutputBuffer() As String
        Get
            Return _outputBuffer
        End Get
        Set(ByVal value As String)
            _outputBuffer = value
        End Set
    End Property


End Class 'Usb2DynamixelWriteArgs

#End Region

#Region " DynamixelReadArgs "

Public Class DynamixelReadArgs : Inherits EventArgs

    Private _inputBuffer As String = Nothing


    Public Sub New(ByVal buff As Byte())
        If Not buff Is Nothing AndAlso buff.Length > 0 Then
            For Each b As Byte In buff
                'Format output
                _inputBuffer += IIf(Hex(b).Length = 1, "0" & Hex(b), Hex(b)) & " "
            Next
        End If
    End Sub


    Public Property InputBuffer() As String
        Get
            If Not _inputBuffer Is Nothing AndAlso _inputBuffer.Length > 0 Then Return _inputBuffer
            Return String.Empty
        End Get
        Set(ByVal value As String)
            _inputBuffer = value
        End Set
    End Property

End Class 'Usb2DynamixelReadArgs

#End Region

#Region " DynamixelControlTableArgs "

Public Class DynamixelControlTableArgs : Inherits EventArgs

    Private _id As Integer
    Private _register As Dynamixel.DynamixelRegister
    Private _args() As Byte


#Region " Constructor "

    Public Sub New(ByVal id As Integer, ByVal register As Dynamixel.DynamixelRegister, ByVal ParamArray args() As Byte)
        _id = id
        _register = register
        _args = args
    End Sub

#End Region


#Region " Properties "

    Public ReadOnly Property ID() As Integer
        Get
            Return _id
        End Get
    End Property


    Public ReadOnly Property Register() As Dynamixel.DynamixelRegister
        Get
            Return _register
        End Get
    End Property


    Public ReadOnly Property Args() As Byte()
        Get
            Return _args
        End Get
    End Property

#End Region



End Class

#End Region